/*
  Kalman.h : SiGN-SSM - Gene Network Estimation with State Sapce Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida
	Masao Nagasaki, and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.


  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#ifndef __KALMAN_H
#define __KALMAN_H

#include "SSM.h"

double Kalman_filter(SSM * ssm, SSMState * X, int p, int k, int N,
					 double * Y, int * table, int ** missing,  SSMWork * work);
void Kalman_smoother(SSMState * X, int k, int N, double * w, double * W1, double * W2);
void Kalman_lagone_cov(SSMState * X, int k, int N);
void Kalman_apply_output_each(SSM * ssm, int k, SSMData * Y, SSMState * X, SSMWork * work,
							  const char * prefix, const char * postfix, int id, int pv);
void Kalman_apply_output(SSM * ssm, int k, SSMData * Y, SSMState * X, SSMWork * work,
						 const char * prefix, const char * postfix, int id, int pv);
void Kalman_infer_mu0(SSM * ssm, SSMState * X, int k, SSMData * Y, SSMWork * work);

#endif /* __KALMAN_H */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo       */
