/*
  SSMInit.c : SiGN-SSM - Gene Network Estimation by State Space Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida,
	Masao Nagasaki and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  
  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#include "SSMInit.h"
#include <math/SGN_Stat.h>
#include <math/SGN_RNG.h>

#include <stdio.h>
#include <stdlib.h>

#if defined(USE_FUJITSU_FCC_SSL)
#include <cssl.h>
#include "math/SGN_Matrix.h"
#elif defined(USE_INTEL_MKL)
#include <mkl_vsl.h>
#endif

/*!\brief Generates SSMInit instance.
 *
 * All the fields of SSMInit are initialized by the default values.
 *
 *\return pointer to the SSMInit instance initialized by the default values.
 *
 *<DIV align="right"><A href="#top">Go to Top</A></DIV>
 *\memberof SSMInit */
SSMInit * SSMInit_new(){
  SSMInit * init = malloc(sizeof(SSMInit));

  init->H_mean =  0.0;
  init->H_SD   =  1.0;
  init->F_min  = -1.5;
  init->F_max  =  1.5;
  init->mu0    =  0.0;
  init->SD0    =  1.0;
  init->rng    =  SGN_RNG_new();
  return init;
}
/* SSMInit_new() =========================================================== */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo.      */
