/*
  SSMInit.h : SiGN-SSM - Gene Network Estimation with State Space Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida,
	Masao Nagasaki and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  
  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#ifndef __SSM_INIT_H
#define __SSM_INIT_H

#include <math/SGN_RNG.h>

/*!\brief Structure of parameters used to generate the initial model parameters.
 *
 *\sa SSM_init() */
typedef struct {
  /*!\brief Mean for generating H. (element-wise Gaussian) */
  double H_mean;

  /*!\brief Standard deviation for generating H. Not used in the current implementation.
   *    (element-wise Gauusian)  */
  double H_SD;

  /*!\brief Lower bound of F. (uniform distribution) */
  double F_min;

  /*!\brief Upper bound of F. (uniform distribution) */
  double F_max;

  /*!\brief Mean mu0. (diagonal-element-wise Gaussian) */
  double mu0;

  /*!\brief S.D sd0. (diagonal-element-wise Gaussian) */
  double SD0;

  /*!\brief Random seed */
  int seed;

  SGN_RNG * rng;

} SSMInit;
/* typedef InitParams ====================================================== */

SSMInit * SSMInit_new();

#endif /* __SSM_INIT_H */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo.      */
