/*
  SSMResult.h : SiGN-SSM - Gene Network Estimation with State Sapce Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida,
	Masao Nagasaki and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  
  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#ifndef __SSM_RESULT_H
#define __SSM_RESULT_H

#include "SSM.h"

/*!\brief Structur for storing an SSM estimation result returned by SSM_EM_algo() and
 *  SSM_EM_iterate().
 *
 *\sa SSM, SSM_EM, SSM_EM_algo(), SSM_EM_iterate() */
typedef struct {
  /*!\brief Number of objects. */
  int p;

  /*!\brief Size of dimension */
  int dim;

  /*!\brief Log likelihood. */
  double likelihood;

  /*!\brief BIC (Bayesian Information Criterion */
  double bic;

  /*!\brief Whether the likelihood increased monotone. */
  int monotone;

  /*!\brief Whether the EM algorithm is converged. */
  int converged;

  /*!\brief Number of loops until converged. */
  int loops;

  /*!\brief Estimated model parameters. */
  SSM * ssm;

  /*!\brief (\a k, \a p)-object-module projection matrix */
  double * D;

  /*!\brief (\a k)-diagonal matrix. */
  double * L;
} SSMResult;

void SSMResult_alloc(SSMResult * result, int p, int dim);
void SSMResult_free(SSMResult * result);
void SSMResult_output(SSMResult * result, const char * prefix, const char * postfix, int id);
void SSMResult_output_each(SSMResult * result, const char * prefix, const char * postfix, int id);
void SSMResult_output_summary_fp(SSMResult * result, int id, FILE * fp);
void SSMResult_calc_D_L(SSMResult * result, SSMWork * work);
SSMResult SSMResult_read_fp(FILE * fp);
void SSMResult_calc_Psi(SSMResult * result, double * Psi, double * W);

#endif /* __SSM_RESULT_H */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo       */
