/*
  SSM_EM.h : SiGN-SSM - Gene Network Estimation with State Sapce Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida,
	Masao Nagasaki and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  
  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#ifndef __SSM_EM_H
#define __SSM_EM_H

#include "SSM.h"
#include "SSMData.h"
#include "SSMResult.h"

/*!\brief Structure for parameters of the EM algorithm.
 *
 *\sa SSM, SSMState, SSMSuffStat, SSMData, SSMResult, SSMInit, SSMWork, \ref Kalman
 */
typedef struct {

  /*!\brief Condition of convergence. */
  double converge;

  /*!\brief EM loop limit. */
  int    loops;

  /*!\brief R=rI or diagonal */
  int    R_rI;

  /*!\brief Updates mu0 or not */
  int    update_mu0;

  /*!\brief Abosulte convergence */
  int    abs_converge;

  /*!\brief Ridge */
  double ridge;

  /*!\brief Non-zero if contrain F when updating. */
  int   constrain_F;

  /*!\brief Constrain constant for updating SSM::F. */
  double g;

  /*!\brief Maximum retry count when the estimation failed. -1 for unlimited retry. */
  int retry;

  /*!\brief Verbose level */
  int verbose;
  
} SSM_EM;

/* ========================================================================= */

SSM_EM * SSM_EM_new();
void     SSM_EM_ridge(int k, double ridge, double * Txx, double * Tx1x1);

SSMResult SSM_EM_iterate(SSM_EM * em, SSMInit * init, int iterations,
						 SSMData * Y, int k,
						 SSM * ssm1, SSM * ssm2,
						 SSMState * X, SSMSuffStat * T, SSMSuffStat * sumT,
						 SSMWork * work);

double   SSM_EM_estimate(SSM * ssm, SSMState * X, 
						 SSMSuffStat * T,SSMSuffStat * sumT,
						 SSMData * Y, int k, double ridge, SSMWork * work);

SSMResult SSM_EM_algo(SSM_EM * em, SSM * ssm, SSMState * X, 
					  SSMSuffStat * T, SSMSuffStat * sumT, 
					  SSMData * Y, int k, SSMWork * work);

void SSM_EM_update(SSM * ssm, SSMSuffStat * T,
				   int p, int k, int m, int samples,
				   int mu0_update, int unique_r, 
				   double * C, double g,
				   SSMWork * work);

void SSM_EM_set_C(int k, double * C);

#endif /* __SSM_EM_H */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo       */
