/*
  TXTS.c : TXTS Simple Text network file format

  Copyright (C) 2010, 2014 SiGN Project members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#include "TXTS.h"
#include "../util/SGN_Str.h"
#include <string.h>
#include <strings.h>

void TXTS_add_node(SGN_Network * network, int index){
  char name_buff[100];
  while(SGN_Network_num_nodes(network) <= index){
	sprintf(name_buff, "G%d", (int) (SGN_Network_num_nodes(network) + 1));
	SGN_Node * node = SGN_Node_new(name_buff, name_buff);
	SGN_Network_add_node(network, node);
  }
}

SGN_Network * TXTS_read_fp(FILE * fp, SGN_KeyValues * args){
  char * buff = NULL;
  size_t buff_size = 0;
  SGN_Network * network = SGN_Network_new();

  fprintf(SIGN_ERR, "Reading TXTS network file...\n");
  int r;
  int count = 0;
  while((r = SGN_Str_readline(fp, &buff, &buff_size)) == 0){
	if(strcmp(buff, "-") == 0) break; /* break line */

	char * pos = index(buff, '\t');
	int child = atoi(buff);
	if(pos == NULL){
	  TXTS_add_node(network, child);
	  continue;
	}
	int max = child;
	
	while(pos != NULL){
	  int parent = atoi(pos + 1);
	  max = parent > max ? parent : max;
	  
	  /* Add required nodes */
	  TXTS_add_node(network, max);
	  SGN_Network_add_edge_index(network, parent, child);
	  count++;
	  pos = index(pos + 1, '\t');
	}
	
  }/* while() */
  fprintf(SIGN_ERR, "  Read %d nodes, %d edges.\n",
		  (int) SGN_Network_num_nodes(network), count);
  return network;
}
/* TXTS_read_fp() ========================================================== */

int TXTS_count_edges(SGN_StrArray * files, double ** graph){
  int count = 0;
  char * buff = NULL;
  size_t buff_size = 0;

  for(int i = 0; i < SGN_StrArray_size(files); i++){
	const char * file = SGN_StrArray_get(files, i);
	fprintf(SIGN_ERR, "Processing: %s\n", file); fflush(SIGN_ERR);

	FILE * fp = fopen(file, "r");
	if(fp == NULL) continue;

	while(!feof(fp)){
	  int edges = 0, nodes = 0, r;
	  
	  while((r = SGN_Str_readline(fp, &buff, &buff_size)) == 0){
		if(strcmp(buff, "-") == 0) break; /* break line */
		nodes++;
		
		char * pos = strchr(buff, '\t');
		int child = strtol(buff, NULL, 10);
		if(pos == NULL) continue;

		while(pos != NULL){
		  int parent = strtol(pos + 1, NULL, 10);
		  
		  graph[parent][child] += 1.0;
		  
		  edges++;
		  pos = strchr(pos + 1, '\t');
		}
	  }/* while() */
	  
	  count++;
	  fprintf(SIGN_ERR, "  Read %d nodes, %d edges.\n", nodes, edges);
	} /* while(!feof(fp)); */
	fclose(fp);
  } /* for i < |files| */
  return count;
}
/* TXTS_count_edges() ====================================================== */

/* End of file.  Copyright (C) 2010, 2014  SiGN Project Members */
