/*
  lang/SGN_Object.h : Generic Object

  Copyright (C) 2009-2013 SiGN Project Members.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_OBJECT_H
#define __SGN_OBJECT_H

#include "SGN_Type.h"

typedef struct SGN_Object_t SGN_Object;

SGN_Object * SGN_Object_new();
SGN_Object * SGN_Object_new_value(void * ptr, SGN_Type type);
SGN_Object * SGN_Object_new_int(int value);
SGN_Object * SGN_Object_new_double(double value);
SGN_Object * SGN_Object_new_string(const char * value);
SGN_Object * SGN_Object_new_error(int value);
SGN_Object * SGN_Object_new_by_string(const char * value, SGN_Type type);

void         SGN_Object_delete(SGN_Object * this);
void         SGN_Object_delete_v(void * this);

SGN_Object * SGN_Object_clone(const SGN_Object * this);

void * SGN_Object_get(SGN_Object * this, SGN_Type type);
void * SGN_Object_ptr(const SGN_Object * this);

void     SGN_Object_set(SGN_Object * this, void * pointer, SGN_Type type);
void *   SGN_Object_set_ret(SGN_Object * this, void * pointer, SGN_Type type);

void *       SGN_Object_pointer(const SGN_Object * this);
SGN_Type     SGN_Object_type(const SGN_Object * this);
const char * SGN_Object_type_name(const SGN_Object * this);

int    SGN_Object_int_value(const SGN_Object * this, int def_value);
double SGN_Object_double_value(const SGN_Object * this, double def_value);
const char * SGN_Object_string_value(const SGN_Object * this, const char * def_value);

void         SGN_Object_fprint(const SGN_Object * this, FILE * fp);
void         SGN_Object_fprint_CSML_type(const SGN_Object * this, FILE * fp);

#endif /* __SGN_OBJECT_H */

/* End of file.  Copyright (C) 2010-2013  SiGN Project Members */
