/*
  lang/SGN_ObjectI.h : Interface for Generic Object

  Copyright (C) 2011 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_OBJECT_I_H
#define __SGN_OBJECT_I_H

#include "SGN_Type.h"

#ifdef DOXY
typedef struct SGN_ObjectI {
#else
typedef struct SGN_ObjectI_t SGN_ObjectI;
struct SGN_ObjectI_t {  
#endif
  SGN_Type type;
  /*!\brief Deletes the instance.
   *
   */
  void (*delete)(SGN_ObjectI *);

  /*!\brief Generates the clone of the instance.
   *
   */
  SGN_ObjectI * (*clone)(const SGN_ObjectI *);
};

/*!\struct SGN_ObjectI
 *\brief Interface for generic objects.
 *
 *
 * <h3>Inherited interfaces</h3>
 * - SGN_GraphI
 *  - Graph, AdjGraph
 *
 *\author Yoshinori Tamada &lt;tamada at is.s.u-tokyo.ac.jp&gt;
 */

/*!\brief Deletes the instance.
 *\memberof SGN_ObjectI
 */													
#define SGN_ObjectI_delete(o) ((SGN_ObjectI *)o)->delete((SGN_ObjectI *)o)

#define SGN_ObjectI_clone(o) ((SGN_ObjectI *)o)->clone((const SGN_ObjectI *)o)
  
#endif /* __SGN_OBJECT_I_H */
  
/* End of file.  Copyright (C) 2011  SiGN Project Members */
