/*
  math/OTO_PK.h : Online Topological Ordering by PK Algorithm.

  Copyright (C) 2009 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __OTO_PK_H
#define __OTO_PK_H

typedef struct {
  int id;
  int ord;
} OTO_PK_sort_t;

typedef struct {
  int size;
  int * ord;
  int * visited;
  OTO_PK_sort_t * Rf;
  int Rfp;
  OTO_PK_sort_t * Rb;
  int Rbp;
} OTO_PK;

OTO_PK * OTO_PK_new(int size);
OTO_PK * OTO_PK_new_pcgraph(SGN_PCGraph * g);
void     OTO_PK_delete(void * this);
void     OTO_PK_copy(const OTO_PK * this, OTO_PK * oto);
int      OTO_PK_add_edge(OTO_PK * this, int x, int y, SGN_PCGraph * g);
int      OTO_PK_check_cyclic(OTO_PK * this, int x, int y, SGN_PCGraph * g);
int      OTO_PK_check_cyclic_reorder(OTO_PK * this, int x, int y, SGN_PCGraph * g);
void     OTO_PK_print_order(OTO_PK * this, FILE * fp);

#endif /* __OTO_PK_H */

/* End of file.  Copyright (C) 2009  SiGN Project Members */
