/*
  math/SGN_AdjGraph.h : Graph structure by an adjacency matrix.

  Copyright (C) 2011,2013 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_ADJ_GRAPH_H
#define __SGN_ADJ_GRAPH_H

#include "SGN_GraphI.h"

/*!\brief Directed graph structure by an adjacency matrix.
 *
 * This is an implementation of SGN_GraphI interface type.
 *
 * SGN_AdjGraph is very fast at
 *  _ Addition and Removal of an edge.
 *  - Checks if a certain edge exists.
 *
 *\sa SGN_GraphI
 *\author Yoshinori Tamada &lt;tamada at is.s.u-tokyo.ac.jp&gt;
 */
typedef struct {
  SGN_GraphI graph;

  size_t nodes;

  /*!\brief pointer to the adjacency matrix.
   *
   * (\a u, \a v) edge is stored at \a matrix[u + v * \a size].
   *
   */
  char * matrix;
  
} SGN_AdjGraph;

SGN_AdjGraph * SGN_AdjGraph_new();
SGN_AdjGraph * SGN_AdjGraph_news(size_t nodes);
SGN_ObjectI * SGN_AdjGraph_clone(const SGN_ObjectI * obj);
void SGN_AdjGraph_delete(SGN_ObjectI * g);

void SGN_AdjGraph_add_edge(SGN_GraphI * g, int u, int v);
void SGN_AdjGraph_remove_edge(SGN_GraphI * gg, int u, int v);
int  SGN_AdjGraph_check_edge(const SGN_GraphI * g, int u, int v);
void SGN_AdjGraph_print_name(const SGN_GraphI * gg, FILE * fp, const SGN_StrArray * names);
void SGN_AdjGraph_print_name2(const SGN_GraphI * gg, FILE * fp, const SGN_StrArray * names);

void SGN_AdjGraph_complete(SGN_AdjGraph * g);

SGN_GraphI * SGN_AdjGraph_to_GraphI(SGN_AdjGraph * g);

#endif /* __SGN_ADJ_GRAPH_H */

/* End of file. Copyright (C) 2011,2013  SiGN Project Members */
