/*
  math/SGN_Graph.h : Graph module for efficient manipulation

  Copyright (C) 2008 - 2011  SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_GRAPH_H
#define __SGN_GRAPH_H

#include "SGN_GraphI.h"

#include "../util/SGN_IntArray.h"
#include "../util/SGN_StrArray.h"

/*!\brief Structure for handling graphs.
 *
 *\author Yoshinori Tamada &lt;tamada AT is.s.u-tokyo.ac.jp
 */
typedef struct {
#ifdef DOXY
private:
#endif
  SGN_GraphI graph;
  size_t nodes;
  SGN_IntArray * parents;
} SGN_Graph;

SGN_Graph *    SGN_Graph_new(size_t nodes);
SGN_IntArray * SGN_Graph_parents(const SGN_Graph * this, int u);
int            SGN_Graph_num_parents(const SGN_Graph * this, int u);
SGN_IntArray * SGN_Graph_gen_children(const SGN_Graph * this, int u);
void           SGN_Graph_set_children_all(const SGN_Graph * this, SGN_IntArray * ar);
size_t         SGN_Graph_num_edges(const SGN_Graph * g);
size_t         SGN_Graph_num_nodes(const SGN_Graph * g);
void           SGN_Graph_delete(void * this);
void           SGN_Graph_add_edge(SGN_Graph * g, int u, int v);
void           SGN_Graph_add_edge_nr(SGN_Graph * g, int u, int v);
SGN_Graph *    SGN_Graph_collect_paths(const SGN_Graph * g,
									   const SGN_IntArray * starts, const SGN_IntArray * ends,
									   int length, int exact);
int            SGN_Graph_check_edge(const SGN_Graph * this, int u, int v);
int            SGN_Graph_check_cyclic(SGN_Graph * this, int u, int v);
int            SGN_Graph_check_cyclic2(const SGN_Graph * this, int u, int v,
									   int j, const SGN_IntArray * j_parents);
void           SGN_Graph_swap_parents(SGN_Graph * this, int u, SGN_IntArray * parents);
void           SGN_Graph_remove_parent_index(SGN_Graph * this, int u, int index);
void           SGN_Graph_clear(SGN_Graph * this);
void           SGN_Graph_copy(const SGN_Graph * this, SGN_Graph * dst);
void           SGN_Graph_show(FILE * fp, const SGN_Graph * this);
void           SGN_Graph_print_name(const SGN_Graph * this, FILE * fp, const SGN_StrArray * names);
void           SGN_Graph_print_name2(const SGN_Graph * this, FILE * fp, const SGN_StrArray * names);
void           SGN_Graph_print_nameI(const SGN_GraphI * g, FILE * fp, const SGN_StrArray * names);
void           SGN_Graph_print_name2I(const SGN_GraphI * g, FILE * fp, const SGN_StrArray * names);
SGN_Graph *    SGN_Graph_convert(const SGN_Graph * this, SGN_IntArray * table, size_t size);

void           SGN_Graph_copy_GraphI(const SGN_GraphI * g, SGN_Graph * dst);
int            SGN_Graph_num_nodesI(const SGN_GraphI * g);
int            SGN_Graph_check_edgeI(const SGN_GraphI * g, int u, int v);
void           SGN_Graph_deleteI(SGN_ObjectI * this);
SGN_GraphI *   SGN_Graph_to_GraphI(SGN_Graph * this);

#ifdef USE_MPI
#include <mpi.h>
void SGN_Graph_Bcast(SGN_Graph ** this, int root, MPI_Comm comm);
#endif /* USE_MPI */

#endif /* __SGN_GRAPH_H */

/* End of file.  Copyright (C) 2008-2011  SiGN Project Members */
