/*
  math/SGN_GraphI.h

  Copyright (C) 2011-2012 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_GRAPH_I_H
#define __SGN_GRAPH_I_H

#include "../lang/SGN_ObjectI.h"
#include "../util/SGN_StrArray.h"

#ifdef DOXY
typedef struct SGN_GraphI {
#else
typedef struct SGN_GraphI_t SGN_GraphI;
struct SGN_GraphI_t {
#endif
  SGN_ObjectI object;

  /*!\brief Returns the number of nodes.
   *
   */
  int (*num_nodes)(const SGN_GraphI * g);
  
  /*!\brief Adds an edge.
   *
   * This adds an edge from \a u to \a v to the graph \a g.
   *
   */
  void (*add_edge)(SGN_GraphI * g, int u, int v);
  
  /*!\brief Checks if the edge exists.
   *
   * This function returns none-zero value if
   * the edge from \a u to \a v exists in the graph \a g,
   * or zero otherwise.
   *
   * If the graph is undirected, the direction of the edge
   * is ignored.
   */
  int (*check_edge)(const SGN_GraphI * g, int u, int v);

  /*!\brief Removes an edge.
   *
   * This removes an edge from \a u to \a v in the graph \a g.
   */
  void (*remove_edge)(SGN_GraphI * g, int u, int v);

  /*!\brief Prints the graph structure with names.
   *
   */
  void (*print_name)(const SGN_GraphI * g, FILE * fp, const SGN_StrArray * names);
  void (*print_name2)(const SGN_GraphI * g, FILE * fp, const SGN_StrArray * names);
};

#define SGN_GraphI_num_nodes(g) g->num_nodes(g)
#define SGN_GraphI_add_edge(g,u,v) g->add_edge(g,u,v)
#define SGN_GraphI_check_edge(g,u,v) g->check_edge(g,u,v)
#define SGN_GraphI_remove_edge(g,u,v) g->remove_edge(g,u,v)
#define SGN_GraphI_print_name(g,f,a) g->print_name(g,f,a)
#define SGN_GraphI_print_name2(g,f,a) g->print_name2(g,f,a)

SGN_ObjectI * SGN_GraphI_to_ObjectI(SGN_GraphI * g);

#endif /* __SGN_GRAPH_I_H */


/*!\struct SGN_GraphI
 *\brief Interface for graphs.
 *
 * This is a subclass of the SGN_ObjectI class.
 *
 *\sa SGN_ObjectI, SGN_AdjGraph
 *\author Yoshinori Tamada &lt;tamada at is.s.u-tokyo.ac.jp&gt;
 */


/* End of file.  Copyright (C) 2011-2012  SiGN Project Members */
