/*
  sign/math/SGN_Math.h : Utility functions of mathematics.

  Copyright (C) 2008-2012 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_MATH_H
#define __SGN_MATH_H

#include <stdio.h>
#include <stdlib.h>

/*!\brief Pi. Circumference ratio to diameter. 
 * 
 *\ingroup Math  */
#define SGN_Math_PI 3.14159265358979323846264

#define SGN_Math_max(x,y)  (x > y ? x : y)
#define SGN_Math_min(x,y)  (x < y ? x : y)

int    SGN_Math_vmaxv(const size_t num, const int * vec);
void   SGN_Math_minmax(const double * ar, const size_t num,
				   double * min, double * max);
double SGN_Math_sum(const double * x, const int n);
int    SGN_Math_isum(const int * ar, const int n);
int    SGN_Math_isum2(const int * ar, const int n, const int inc);
int    SGN_Math_gcd(int x, int y);
size_t SGN_Math_choose(int n, int c);
size_t SGN_Math_choose2(int n, int k);
int    SGN_Math_combo(int n, int c, int * ar);
void   SGN_Math_combo_index_table(int n, int c, size_t * table);
size_t SGN_Math_combo_index(int n, int c, const int * ar, 
							const size_t * table, int tn);
size_t SGN_Math_combo_index2(int n, int c, const int * ar,
							 const size_t * table, int tn, const int * ar2);
size_t SGN_Math_combo_index3(int n, int c, const int * ar,
							 const size_t * table, int tn, const int * ar2, const int * ar3);
void   SGN_Math_index_combo(int n, int c, size_t index, int * ar, 
							const size_t * table, int tn);
void SGN_Math_index_combo2(int n, int c, size_t index, int * ar, 
						   const size_t * table, int tn, const int * ar2);

#endif /* __SGN_MATH_H */

/* End of file.  Copyright (C) 2008-2012  SiGN Project Members */
