/*
  sign/math/SGN_Matrix.h : Matrix and LAPACK wrapper functions

  Copyright (C) 2008-2012 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_MATRIX_H
#define __SGN_MATRIX_H

#include <stdlib.h>
#include <stdio.h>
#ifndef __SIGN_SSM
#include "../util/SGN_Memory.h" /* Do not include for SiGN-SSM */
#endif

/* Convenient predefined constatns */
extern int    SGN_Matrix_INT_ONE;
extern int    SGN_Matrix_INT_ZERO;
extern double SGN_Matrix_DOUBLE_ONE;
extern double SGN_Matrix_DOUBLE_ZERO;
extern double SGN_Matrix_DOUBLE_MINUS_ONE;/* Minus One (-1)*/

/* ------------------------------------------------------------------------- */
/*  Mac OS X  vecLib                                                         */
/* ------------------------------------------------------------------------- */
#if defined(USE_MACOSX_VECLIB)

#include <Accelerate/Accelerate.h>

typedef __CLPK_integer SGN_Matrix_int;
typedef __CLPK_doublereal SGN_Matrix_double;

#ifndef USE_CBLAS
#define USE_CBLAS
#endif

/* LAPACK Routines */
extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;
extern char * SGN_Matrix_L_NO_TRANS;
extern char * SGN_Matrix_L_TRANS;
extern char * SGN_Matrix_LA;
extern char * SGN_Matrix_LO;

#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,LW,IF) dsytrf_(UL,N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_(UL,N,A,LA,IP,W,IF)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) dsyev_(JB,UL,N,A,LA,W,WO,LW,IF)
#define SGN_Matrix_dgesv(N,M,A,LDA,IP,B,LDB,IF) dgesv_(N,M,A,LDA,IP,B,LDB,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_(M,N,A,LA,IP,IF)
#define SGN_Matrix_dgetri(N,A,LA,IP,W,LW,IF) dgetri_(N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dgetrs(TA,N,M,A,LA,IP,B,LB,IF) dgetrs_(TA,N,M,A,LA,IP,B,LB,IF)
#define SGN_Matrix_dgesvd(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF) \
                                  dgesvd_(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF)

/* USE_MACOSX_VECLIB */

/* ------------------------------------------------------------------------- */
/*  Solaris  Sun Performance Library  64-bit                                 */
/* ------------------------------------------------------------------------- */
#elif defined(USE_SUNPERF64)

#include <sunperf.h>

typedef long SGN_Matrix_int;
typedef double SGN_Matrix_double;
#define SGN_Matrix_T 'T'
#define SGN_Matrix_N 'N'
#define SGN_Matrix_C 'C'

#define SGN_Matrix_NO_TRANS 'N'
#define SGN_Matrix_TRANS    'C'

#define SGN_Matrix_U  'U'
#define SGN_Matrix_L  'L'

extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;

/* BLAS Routines */
/*   Level 1 BLAS */
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy_64(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot_64(N,X,IX,Y,IY)
#define SGN_Matrix_dnrm2(N,X,IX) dnrm2_64(N,X,IX)
#define SGN_Matrix_dasum(N,X,IX) dasum_64(N,X,IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                    dgemv_64(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                                     dsymv_64(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)

/*   Level 3 BLAS */
#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                               dgemm_64(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC)
#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,LW,IF) dsytrf_64_(UL,N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_64_(UL,N,A,LA,IP,W,IF)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                      dsyrk_64(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) \
                                            dsyev_64_(JB,UL,N,A,LA,W,WO,LW,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_64_(M,N,A,LA,IP,IF)

/* USE_SUNPERF64 */

/* ------------------------------------------------------------------------- */
/*  Solaris  Sun Performance Library  32-bit                                 */
/* ------------------------------------------------------------------------- */
#elif defined(USE_SUNPERF32)

#include <sunperf.h>

typedef int SGN_Matrix_int;
typedef double SGN_Matrix_double;

#define SGN_Matrix_T 'T'
#define SGN_Matrix_N 'N'
#define SGN_Matrix_C 'C'

#define SGN_Matrix_NO_TRANS 'N'
#define SGN_Matrix_TRANS    'C'

#define SGN_Matrix_U  'U'
#define SGN_Matrix_L  'L'
extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;

/* BLAS Routines */
/*   Level 1 BLAS */
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot(N,X,IX,Y,IY)
#define SGN_Matrix_dnrm2(N,X,IX) dnrm2(N,X,IX)
#define SGN_Matrix_dasum(N,X,IX) dasum(N,X,IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                       dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                           dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)

/*   Level 3 BLAS */
#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                 dgemm_(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC)

/* LAPACK Routines */
#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,IW,IF) dsytrf_(UL,N,A,LA,IP,W,IW,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_(UL,N,A,LA,IP,W,IF)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                        dsyrk_(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) dsyev_(JB,UL,N,A,LA,W,WO,LW,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_(M,N,A,LA,IP,IF)

/* ------------------------------------------------------------------------- */
/*   IBM ESSL                                                                */
/* ------------------------------------------------------------------------- */
#elif defined(USE_IBM_ESSL)

#include <essl.h>

typedef int SGN_Matrix_int;
typedef double SGN_Matrix_double;

extern char * SGN_Matrix_U;
extern char * SGN_Matrix_L;
extern char * SGN_Matrix_T;
extern char * SGN_Matrix_N;

extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LN;

void dgetrf_(int*,int*,double*,int*,int*,int*);
void dgetri_(int* n, double* a, int* lda, int * ipv,
			 double* work, int* lwork, int*);

/* BLAS ROUTINES */
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot(N,X,IX,Y,IY)
#define SGN_Matrix_dasum(N,X,IX) dasum(N,X,IX)
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                      dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC)
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                       dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                                      dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)

#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                 dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC)

/* LAPACK ROUTINES */
#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,IW,IF) dgetrf_(N,N,A,LA,IP,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF)    dgetri_(N,A,LA,IP,W,IF)


/* ------------------------------------------------------------------------- */
/*   INTEL MKL                                                               */
/* ------------------------------------------------------------------------- */
#elif defined(USE_INTEL_MKL)

#include <mkl_blas.h>
#include <mkl_lapack.h>

typedef MKL_INT SGN_Matrix_int;
typedef double  SGN_Matrix_double;

extern char * SGN_Matrix_T;
extern char * SGN_Matrix_N;
extern char * SGN_Matrix_C;
extern char * SGN_Matrix_NO_TRANS;
extern char * SGN_Matrix_TRANS;
extern char * SGN_Matrix_U;
extern char * SGN_Matrix_L;

/* CBLAS ROUTINE */
/*   Level 1 BLAS */
#define SGN_Matrix_dscal(N,ALPHA,X,IX) dscal(N,ALPHA,X,IX)
#define SGN_Matrix_dcopy(N,X,IX,Y,IY) dcopy(N,X,IX,Y,IY)
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot(N,X,IX,Y,IY)
#define SGN_Matrix_dnrm2(N,X,IX) dnrm2(N,X,IX)
#define SGN_Matrix_dasum(N,X,IX) dasum(N,X,IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                        dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                                          dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dger(M,N,ALPHA,X,IX,Y,IY,A,LDA)  dger(M,N,ALPHA,X,IX,Y,IY,A,LDA)

/*   Level 3 BLAS */
#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                   dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC)

#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                          dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC)

/* LAPACK ROUTINE */
extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;
extern char * SGN_Matrix_L_NO_TRANS;
extern char * SGN_Matrix_L_TRANS;
extern char * SGN_Matrix_LA;
extern char * SGN_Matrix_LO;

#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,LW,IF) dsytrf_(UL,N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_(UL,N,A,LA,IP,W,IF)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) dsyev_(JB,UL,N,A,LA,W,WO,LW,IF)
#define SGN_Matrix_dgesv(N,M,A,LDA,IP,B,LDB,IF) dgesv_(N,M,A,LDA,IP,B,LDB,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_(M,N,A,LA,IP,IF)
#define SGN_Matrix_dgetri(N,A,LA,IP,W,LW,IF) dgetri_(N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dgetrs(TA,N,M,A,LA,IP,B,LB,IF) dgetrs_(TA,N,M,A,LA,IP,B,LB,IF)
#define SGN_Matrix_dgesvd(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF) \
                                  dgesvd_(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF)


/* ------------------------------------------------------------------------- */
/*  FUJITSU FCC/SSL                                                          */
/* ------------------------------------------------------------------------- */
#elif defined(USE_FUJITSU_FCC_SSL)

#include <fj_lapack.h>

typedef int    SGN_Matrix_int;
typedef double SGN_Matrix_double;

extern char * SGN_Matrix_T;
extern char * SGN_Matrix_N;
extern char * SGN_Matrix_C;
extern char * SGN_Matrix_NO_TRANS;
extern char * SGN_Matrix_TRANS;
extern char * SGN_Matrix_U;
extern char * SGN_Matrix_L;
extern char * SGN_Matrix_LEFT;
extern char * SGN_Matrix_RIGHT;

/* BLAS Routines */
/*   Level 1 BLAS */
#define SGN_Matrix_dscal(N,ALPHA,X,IX) dscal_(N,ALPHA,X,IX)
#define SGN_Matrix_dcopy(N,X,IX,Y,IY) dcopy_(N,X,IX,Y,IY)
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy_(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot_(N,X,IX,Y,IY)
#define SGN_Matrix_dnrm2(N,X,IX) dnrm2_(N,X,IX)
#define SGN_Matrix_dasum(N,X,IX) dasum_(N,X,IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                        dgemv_(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY,1)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                                        dsymv_(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY,1)
#define SGN_Matrix_dger(M,N,ALPHA,X,IX,Y,IY,A,LDA)\
                                        dger_(M,N,ALPHA,X,IX,Y,IY,A,LDA)

/*   Level 3 BLAS */

#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                 dgemm_(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC,1,1)
#define SGN_Matrix_dsymm(LR,UL,M,N,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                 dsymm_(LR,UL,M,N,ALPHA,A,LA,B,LB,BETA,C,LC,1,1)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                 dsyrk_(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC,1,1)

/* LAPACK Routines */
extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;
extern char * SGN_Matrix_L_NO_TRANS;
extern char * SGN_Matrix_L_TRANS;
extern char * SGN_Matrix_LA;
extern char * SGN_Matrix_LO;

#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,IW,IF) dsytrf_(UL,N,A,LA,IP,W,IW,IF,1)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_(UL,N,A,LA,IP,W,IF,1)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) dsyev_(JB,UL,N,A,LA,W,WO,LW,IF,1,1)
#define SGN_Matrix_dgesv(N,M,A,LDA,IP,B,LDB,IF) dgesv_(N,M,A,LDA,IP,B,LDB,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_(M,N,A,LA,IP,IF)
#define SGN_Matrix_dgetri(N,A,LA,IP,W,LW,IF) dgetri_(N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dgetrs(TA,N,M,A,LA,IP,B,LB,IF) dgetrs_(TA,N,M,A,LA,IP,B,LB,IF,1)
#define SGN_Matrix_dgesvd(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF) \
                                dgesvd_(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF,1,1)

/* End of FUJITSU FCC/SSL */

/* ------------------------------------------------------------------------- */
/*  OTHER NON SUPPORTED ENVIRONMENTS                                         */
/* ------------------------------------------------------------------------- */
#else
/* For other environments, we assume FOTRAN BLAS/LAPACK */

#include "SGN_lapack.h"

typedef int SGN_Matrix_int;
typedef double SGN_Matrix_double;

#ifdef USE_CBLAS
#include <cblas.h>
#else
extern char * SGN_Matrix_T;
extern char * SGN_Matrix_N;
extern char * SGN_Matrix_C;

extern char * SGN_Matrix_NO_TRANS;
extern char * SGN_Matrix_TRANS;
extern char * SGN_Matrix_U;
extern char * SGN_Matrix_L;
#endif /* USE_CBLAS */

extern char * SGN_Matrix_LU;
extern char * SGN_Matrix_LL;
extern char * SGN_Matrix_LN;
extern char * SGN_Matrix_LV;
extern char * SGN_Matrix_L_NO_TRANS;
extern char * SGN_Matrix_L_TRANS;
extern char * SGN_Matrix_LA;
extern char * SGN_Matrix_LO;

extern char * SGN_Matrix_LEFT;
extern char * SGN_Matrix_RIGHT;

/* BLAS Routines */
#ifndef USE_CBLAS
/*   Level 1 BLAS */
#define SGN_Matrix_dscal(N,ALPHA,X,IX) dscal_(N,ALPHA,X,IX)
#define SGN_Matrix_dcopy(N,X,IX,Y,IY) dcopy_(N,X,IX,Y,IY)
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) daxpy_(N,ALPHA,X,IX,Y,IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) ddot_(N,X,IX,Y,IY)
#define SGN_Matrix_dnrm2(N,X,IX) dnrm2_(N,X,IX)
#define SGN_Matrix_dasum(N,X,IX) dasum_(N,X,IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                                        dgemv_(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                                          dsymv_(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)
#define SGN_Matrix_dger(M,N,ALPHA,X,IX,Y,IY,A,LDA)  dger_(M,N,ALPHA,X,IX,Y,IY,A,LDA)

/*   Level 3 BLAS */
#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                   dgemm_(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC)
#define SGN_Matrix_dsymm(LR,UL,M,N,ALPHA,A,LA,B,LB,BETA,C,LC) \
                                   dsymm_(LR,UL,M,N,ALPHA,A,LA,B,LB,BETA,C,LC)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                                   dsyrk_(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC)

#endif

/* LAPACK Routines */
#define SGN_Matrix_dsytrf(UL,N,A,LA,IP,W,LW,IF) dsytrf_(UL,N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dsytri(UL,N,A,LA,IP,W,IF) dsytri_(UL,N,A,LA,IP,W,IF)
#define SGN_Matrix_dsyev(JB,UL,N,A,LA,W,WO,LW,IF) dsyev_(JB,UL,N,A,LA,W,WO,LW,IF)
#define SGN_Matrix_dgesv(N,M,A,LDA,IP,B,LDB,IF) dgesv_(N,M,A,LDA,IP,B,LDB,IF)
#define SGN_Matrix_dgetrf(M,N,A,LA,IP,IF) dgetrf_(M,N,A,LA,IP,IF)
#define SGN_Matrix_dgetri(N,A,LA,IP,W,LW,IF) dgetri_(N,A,LA,IP,W,LW,IF)
#define SGN_Matrix_dgetrs(TA,N,M,A,LA,IP,B,LB,IF) dgetrs_(TA,N,M,A,LA,IP,B,LB,IF)
#define SGN_Matrix_dgesvd(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF) \
                                  dgesvd_(JU,JV,M,N,A,LA,S,U,LU,VT,LVT,W,LW,IF)
#endif /* END OF OTHER ENVIRONMENT */

/*  END OF ENVIRONMENT SPECIFIC DEFINITIONS                                  */
/* ------------------------------------------------------------------------- */

/* -- CBLAS ---------------------------------------------------------------- */
#ifdef USE_CBLAS

#define SGN_Matrix_T        CblasTrans
#define SGN_Matrix_N        CblasNoTrans
#define SGN_Matrix_C        CblasConjTrans

#define SGN_Matrix_NO_TRANS CblasNoTrans
#define SGN_Matrix_TRANS    CblasTrans

#define SGN_Matrix_U        CblasUpper
#define SGN_Matrix_L        CblasLower

#define SGN_Matrix_LEFT     CblasLeft
#define SGN_Matrix_RIGHT    CblasRight

#define SGN_Matrix_dscal(N,ALPHA,X,IX) cblas_dscal(*N,*ALPHA,X,*IX)
#define SGN_Matrix_dcopy(N,X,IX,Y,IY) cblas_dcopy(*N,X,*IX,Y,*IY)
#define SGN_Matrix_daxpy(N,ALPHA,X,IX,Y,IY) cblas_daxpy(*N,*ALPHA,X,*IX,Y,*IY)
#define SGN_Matrix_ddot(N,X,IX,Y,IY) cblas_ddot(*N,X,*IX,Y,*IY)
#define SGN_Matrix_dnrm2(N,X,IX) cblas_dnrm2(*N,X,*IX)
#define SGN_Matrix_dasum(N,X,IX) cblas_dasum(*N,X,*IX)

/*   Level 2 BLAS */
#define SGN_Matrix_dgemv(T,M,N,ALPHA,A,LDA,X,IX,BETA,Y,IY)\
                  cblas_dgemv(CblasColMajor,T,*M,*N,*ALPHA,A,*LDA,X,*IX,*BETA,Y,*IY)
#define SGN_Matrix_dsymv(UL,N,ALPHA,A,LA,X,IX,BETA,Y,IY)\
                    cblas_dsymv(CblasColMajor,UL,*N,*ALPHA,A,*LA,X,*IX,*BETA,Y,*IY)
#define SGN_Matrix_dger(M,N,ALPHA,X,IX,Y,IY,A,LDA)\
                    cblas_dger(CblasColMajor,*M,*N,*ALPHA,X,*IX,Y,*IY,A,*LDA)

/*   Level 3 BLAS */
#define SGN_Matrix_dgemm(TA,TB,M,N,K,ALPHA,A,LA,B,LB,BETA,C,LC) \
                  cblas_dgemm(CblasColMajor,TA,TB,*M,*N,*K,*ALPHA,A,*LA,B,*LB,*BETA,C,*LC)
#define SGN_Matrix_dsymm(LR,UL,M,N,ALPHA,A,LA,B,LB,BETA,C,LC) \
                  cblas_dsymm(CblasColMajor,LR,UL,*M,*N,*ALPHA,A,*LA,B,*LB,*BETA,C,*LC)
#define SGN_Matrix_dsyrk(UL,T,N,K,ALPHA,A,LDA,BETA,C,LC) \
                      cblas_dsyrk(CblasColMajor,UL,T,*N,*K,*ALPHA,A,*LDA,*BETA,C,*LC)

#endif /* CBLAS */
/* ------------------------------------------------------------------------- */

SGN_Matrix_double *  SGN_Matrix_new(int m, int n);
SGN_Matrix_double ** SGN_Matrix_new_ar(const int m, const int n, const int k);
void       SGN_Matrix_print(FILE * fp, const double * X, int m, int n);
void       SGN_Matrix_printf(FILE * fp, const double * X, const int m, const int n, 
						 const char * fmt);
void SGN_Matrix_fprintf(FILE * fp, const int digits, 
					const double * X, const int m, const int n);
void       SGN_Matrix_delete(double * M);
void       SGN_Matrix_trans(int M, int N, double * A, double * B);
void       SGN_Matrix_dsycopy(const int n, const double * X, double * Y);
void       SGN_Matrix_dsyge(const int n, double * X);
SGN_Matrix_int SGN_Matrix_dsytrf_work_size(SGN_Matrix_int n);
SGN_Matrix_int SGN_Matrix_dsyev_work_size(SGN_Matrix_int n);
SGN_Matrix_int SGN_Matrix_dgetri_work_size(SGN_Matrix_int n);

double SGN_Matrix_debug_check(FILE * fp, const char * label, const double * X, 
						  const int m, const int n, const int f, const int abs);

#define SGN_Matrix_double_new(x) ((SGN_Matrix_double *) malloc(sizeof(SGN_Matrix_double) * x))

void SGN_Matrix_dgemv_l(const SGN_Matrix_int M, const SGN_Matrix_int N,
					const double alpha, const double * A, const SGN_Matrix_int lda,
					const double * X, const SGN_Matrix_int incX, 
					const double beta, double * Y, const SGN_Matrix_int incY);

void SGN_Matrix_dsyaxpy(const int n, double alpha, double * X, double * Y);

void SGN_Matrix_sy2ge(SGN_Matrix_double * X, const SGN_Matrix_int n);


double SGN_Matrix_trace(const double * X, const SGN_Matrix_int n);

void SGN_Matrix_diag(const int N, double alpha, const double * X, double * A, int LDA);
void SGN_Matrix_dgedm2(int M, int N, double * x, double * A, int LDA);
void SGN_Matrix_dgemd2(int M, int N, double * A, int LDA, double * x);
void SGN_Matrix_dgeidm2(int M, int N, double * x, double * A, int LDA);
void SGN_Matrix_dgemid2(int M, int N, double * A, int LDA, double * x);

void SGN_Matrix_I(const int N, double * X);

double SGN_Matrix_vector_abs_max(int n, const double * x);
double SGN_Matrix_vector_abs_min(int n, const double * x);

#endif /* __SGN_MATRIX_H */

/* End of file.  Copyright (C) 2008-2012  SiGN Project Members  */
