/*
  sign/math/SiGN_lapack.h : C LAPACK Header files for FORTRAN ROUTINE

  Copyright (C) 2008-2010 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SIGN_LAPACK_H
#define __SIGN_LAPACK_H

/* LEVEL 1 BLAS */
void   dscal_(int * N, double * alpha, double * X, int * incX);
void   dcopy_(int * N, double * X, int * incX, double * Y, int * incY);
void   daxpy_(int * N, double * alpha, double * X, int * incX, double * Y, int * incY);
double ddot_(int * N, double * X, int * incX, double * Y, int * incY);
double nrm2_(int * N, double * X, int * incX);
double dasum_(int * N, double * X, int * incX);

/* LEVEL 2 BLAS */
void dgemv_(char * TA, int * M, int * N,
			double * alpha, double * A, int * ldA,
			double * X, int * incX, double * beta, double * Y, int * incY);
void dsymv_(char * UL, int * N, double * alpha, double * A, int * ldA,
			double * X, int * incX, double * beta, double * Y, int * incY);
void dger_(int * M, int * N, double * alpha, double * X, int * incX, 
		   double * Y, int * incY, double * A, int * ldA);

/* LEVEL 3 BLAS */
void dgemm_(char * TA, char * TB, int * M, int * N, int * K,
			double * alpha, double * A, int * ldA, 
			double * B, int * ldB, double * beta, double * C, int * ldC);
void dsymm_(char * LR, char * UL, int * M, int * N, double * alpha, double * A, int * ldA, 
			double * B, int * ldB, double * beta, double * C, int * ldC);
void dsyrk_(char * UL, char * T, int * N, int * K, 
			double * alpha, double * A, int * ldA, 
			double * beta, double * C, int * ldC);

/* LAPACK Routines */

void dsytrf_(char * UL, int * N, double * A, int * ldA, int * IP, double * WORK, int * LWORK, int * info);
void dsytri_(char * UL, int * N, double * A, int * ldA, int * IP, double * W, int * info);
void dsyev_(char * JB, char * UL, int * N, double * A, int * ldA, double * W,
			double * WORK, int * LWORK, int * info);
void dgesv_(int * N, int * M, double * A, int * ldA, int * IP, double * B, int * ldB, int * info);
void dgetrf_(int * M, int * N, double * A, int * ldA, int * IP, int * info);
void dgetri_(int * N, double * A, int * ldA, int * IP, double * WORK, int * LWORK, int * info);
void dgetrs_(char * TA, int * N, int * M, double * A, int * ldA, int * IP, 
			 double * B, int * ldB, int * info);
void dgesvd_(char * JU, char * JV, int * M, int * N, double * A, int * ldA, double * S,
			 double * U, int * ldU, double * VT, int * ldVT, double * WORK, int * LWORK, int * info);

#endif /* __SIGN_LAPACK_H */

/* End of file.  Copyright (C) 2008-2010 SiGN Project Members */
