/*
  signssm.h : SiGN-SSM - Gene Network Estimation with State Sapce Model

  Copyright (C) 2010
    Yoshinori Tamada, Rui Yamaguchi, Seiya Imoto, Osamu Hirose, Ryo Yoshida,
	Masao Nagasaki and Satoru Miyano
      Laboratory of DNA Information Analysis &
	  Laboratory of Sequence Analysis, Human Genome Center,
      The Institute of Medical Science, The Unversity of Tokyo

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  
  Visit <http://sign.hgc.jp/signssm/> for more information.

  Written by Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
  Originally developed by Osamu Hirose, Ryo Yoshida, Seiya Imoto, Rui Yamaguchi,
                          Tomoyuki Higuchi, and Satoru Miyano.
*/

#ifndef __SIGN_SSM_H
#define __SIGN_SSM_H

#include "SSM_EM.h"
#include "util/SGN_Timer.h"
#include "util/SGN_KeyValues.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <strings.h>

/*!\brief Structure for program arguments.
 *
 * Call Args_new() to obtain the instance initialized with default values.
 *
 *\ingroup SiGNSSM */
typedef struct {

  /*!\brief Task ID used to initialize randaom seeds and output IDs. */
  int id;              

  /*!\brief  Random Seed. */
  int seed;            

  /*!\brief Dimensions of state variables to be estimated. */
  int * dims;

  /*!\brief Length of \a dims. */
  int len_dims;

  /*!\brief Number of models (sets) to be estimated for a single size of dimensions. */
  int sets;

  /*!\brief Number of inital values for a single EM algorithm execution. */
  int iteration;

  /*!\brief Output file prefix */     
  char * output_file;

  /*!\brief Output file postfix (extension) */
  char * output_ext;

  /*!\brief Output the matrices/vectors of the estimated model in single files. */
  int output_each;

  /*!\brief Input file name */
  char * input_file;      

  /*!\brief Log file mode */
  int log_mode;      

  /*!\brief SGE mode */
  int sge;

  /*!\brief EM related parameters */
  SSM_EM * em;

  /*!\brief parameters to generate the initial SSM model. */
  SSMInit * init;
  
  /*!\brief Use old implementation using GSL (GNU Scientific Library). */
  int gsl;               

  /*!\brief Execute the test code for debugging. */  
  int debug;

  /*!\brief Rank of this MPI process. */
  int MPI_rank;

  /*!\brief Size of MPI processes. */ 
  int MPI_size;

  /*!\brief File name of the SSM model parameters and
	use them instead of performing the EM algorithm.

	If this is set, SSM::dims, SSM::sets, and SSM::iteration are
	ignored.
  */
  char * ssm_file;

  /*!\brief If zero, skips the final calculation of state vectors
	from the estimated model and output of them. */
  int output_state;

  /*!\brief If non-zero, performs mean shift for each object
   *    before starting the estimation. */
  int mean_shift;

  /*!\brief If non-zero, add the number of CPU processes (cores)
   *    to the output file.
   *
   *  A comma, a caiptal <tt>P</tt>, and the 4 digit, 0-filled letters
   *  is added to the output file preix. E.g. &quot;<tt>.P0128</tt>&quot;
   */
  int add_proc_file;

  /*!\brief Number of threads if the multi-thread (OpenMP) is enabled.  */
  int threads;

  /*!\brief Input file for permutation test */
  char * perm_file;

  /*!\brief Parameters for compiling the permutation results. */
  SGN_KeyValues * ssmperm;

  /*!\brief Output file name of the compiled permuation result */
  char * ssmtxt;

  /*!\brief If non-zero, calculate and output p-values. */
  int output_pvalues;

#ifdef USE_EDF
  char * input_args;
#endif
} Args;

void ssm_output(SSMResult * result, SSMWork * work, SSMState * X, SSMData * Y,
				const char * prefix, const char * postfix, int id, int each, int state, int pv);

#endif /* __SIGN_SSM_H */

/* End of file.  Copyright (C) 2010  HGC, IMS, The University of Tokyo       */
