/*
  SGN_BS.h : Bootstrap related routines

  Copyright (C) 2010-2011, 2014  SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_BS_H
#define __SGN_BS_H

#define SGN_BS_BUFF_SIZE 256

#include "SGN_Network.h"
#ifndef __IO_H
typedef SGN_Network * (* IOReader)(FILE *, SGN_KeyValues *);
#endif

/*!\brief Bootstrap utility functions
 *
 *\author Yoshinori Tamada &lt;tamada at is.s.u-tokyo.ac.jp&gt;
 */
typedef struct {
  int nodes;
  
  double ** graph;
  double *** ud;
  double ** gain;

  char * label_prob;  /* BS.Prob */
  char * label_gain;  /* BS.Gain */
  char * label_type;  /* EdgeType */
  char * label_ud;    /* BS.up/down */
  char * label_ratio; /* BS.EdgeTypeRatio */
  char * label_dir;  /* BS.direction */
  char * label_score; /* edgeScore */

  char * label_hubness;   /* hubness */
  double * hubindex;
  char * label_hubindex; /* BS.HubIndex */

  SGN_Network * fnet;
  SGN_Data * data;

  int one_minus;
  int verbose;
} SGN_BS;

SGN_BS * SGN_BS_new(int size);
void     SGN_BS_delete(SGN_BS * bs);

void SGN_BS_set_verbose(SGN_BS * bs, int verbose);
void SGN_BS_set_SSM(SGN_BS * bs);
void SGN_BS_set_network(SGN_BS * bs, SGN_Network * network);
void SGN_BS_set_Data(SGN_BS * bs, SGN_Data * data);

SGN_StrArray * SGN_BS_gen_files(const char * prefix, const char * suffix, 
						int digits,	int bg, int ed);
SGN_Network * SGN_BS_first_network(SGN_StrArray * filenames, IOReader reader);
int SGN_BS_process(SGN_BS * bs, const SGN_Network * network, 
					const char * freq_key, const char * ud_key, const char * total_key);
void SGN_BS_process_serialized(SGN_BS * bs, void * buff, size_t size);

int SGN_BS_process_multi(SGN_BS * bs, IOReader reader, const char * file, 
					 const char * freq_key, const char * ud_key, const char * total_key);
int SGN_BS_process_files(SGN_BS * bs, IOReader reader, SGN_StrArray * files);
int SGN_BS_process_files_dist(SGN_BS * bs, IOReader reader, SGN_StrArray * files,
						  const char * freq_key, const char * ud_key, const char * total_key);
SGN_Network * SGN_BS_compile_dynamic(SGN_BS * bs, double th, int total, int cmp);
SGN_Network * SGN_BS_compile(SGN_BS * bs, double th, int total, int cmp, int dynamic);

#endif /* __SGN_BS_H */

/* End of file.  Copyright (C) 2010-2011, 2014  SiGN Project Members */

