/*
  sign/util/SGN_DoubleArray.h : Expandable array for double values.

  Copyright (C) 2008-2010 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_DOUBLE_ARRAY_H
#define __SGN_DOUBLE_ARRAY_H

#include <stdio.h>

/*!\brief Expandable array for double values.
 *
 * SGN_DoubleArray module provides functions to realize the automatically
 * expandable data container for double values.
 *
 * Do not use the SGN_DoubleArray structure directly.  To use SGN_DoubleArray 
 * module, use SGN_DoubleArray_new() to generate the SGN_DoubleArray instance, and
 * use SGN_DoubleArray_delete() to release the memory used by the instance when 
 * it is no longer needed.
 *
 *\author Yoshinori Tamada <tamada at ims.u-tokyo.ac.jp>
 */
typedef struct {
#ifdef DOXY
private:
#endif
  double * ptr;
  size_t size;
  size_t buff_size;
} SGN_DoubleArray;

#define SGN_DoubleArray_ERROR_VALUE ((size_t) -1)

SGN_DoubleArray * SGN_DoubleArray_new(void);
SGN_DoubleArray * SGN_DoubleArray_news(size_t size);
SGN_DoubleArray * SGN_DoubleArray_new_ar(size_t ar_size, size_t init_size);
void              SGN_DoubleArray_delete(SGN_DoubleArray * this);
void              SGN_DoubleArray_delete_v(void * this);
SGN_DoubleArray * SGN_DoubleArray_copy(const SGN_DoubleArray * this);
SGN_DoubleArray * SGN_DoubleArray_expand(SGN_DoubleArray * this, size_t length);
SGN_DoubleArray * SGN_DoubleArray_new_str(const char * str);
size_t            SGN_DoubleArray_size(const SGN_DoubleArray * this);
SGN_DoubleArray * SGN_DoubleArray_add(SGN_DoubleArray * this, double value);
SGN_DoubleArray * SGN_DoubleArray_set(SGN_DoubleArray * this, size_t index, double value);
double            SGN_DoubleArray_get(const SGN_DoubleArray * this, size_t index);
size_t            SGN_DoubleArray_find(const SGN_DoubleArray * this, double value);
double *          SGN_DoubleArray_ptr(SGN_DoubleArray * this);
void              SGN_DoubleArray_set_str(SGN_DoubleArray * this, const char * str);
SGN_DoubleArray * SGN_DoubleArray_clear(SGN_DoubleArray * this);
double            SGN_DoubleArray_get_min(const SGN_DoubleArray * this);
void *            SGN_DoubleArray_read(const SGN_DoubleArray * this, size_t index, void * mem);
void              SGN_DoubleArray_fprint(const SGN_DoubleArray * da, FILE * fp);
void              SGN_DoubleArray_fprint_v(const void * da, FILE * fp);

#endif /* __SGN_DOUBLE_ARRAY_H */

/* End of file.  Copyright (C) 2010 SiGN Projet Members */
