/*
  SGN_IntQueue.h : Expandable Integer Queue

  Copyright (C) 2008-2010  SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_INT_QUEUE_H
#define __SGN_INT_QUEUE_H

#include <stdio.h>
#include <stdlib.h>

/*!\brief Expandable Integer Queue
 *
 *\author Yoshinori Tamada &lt;tamad at ims.u-tokyo.ac.jp&gt;
 */
typedef struct {
  int * ptr;
  size_t buff_size;

  size_t pos;
  size_t size;

} SGN_IntQueue;

SGN_IntQueue * SGN_IntQueue_new();
SGN_IntQueue * SGN_IntQueue_new_ar(size_t n);
void           SGN_IntQueue_delete(void * this);
void           SGN_IntQueue_delete_ar(void * this, size_t n);
SGN_IntQueue * SGN_IntQueue_expand(SGN_IntQueue * this, size_t size);
size_t         SGN_IntQueue_size(const SGN_IntQueue * this);
int            SGN_IntQueue_is_empty(const SGN_IntQueue * this);
void           SGN_IntQueue_clear(SGN_IntQueue * this);
void           SGN_IntQueue_enqueue(SGN_IntQueue * this, int value);
int            SGN_IntQueue_dequeue(SGN_IntQueue * this, int * v);
int            SGN_IntQueue_get(const SGN_IntQueue * this, size_t index);
void           SGN_IntQueue_fprint(const SGN_IntQueue * this, FILE * fp);

#endif /* __SGN_INT_QUEUE_H */

/* End of file.  Copyright (C) 2008-2010  SiGN Project Members */
