/*
  util/SGN_KeyObjects.h : Key and its associated objects.
  
  Copyright (C) 2010 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
 */

#ifndef __KEY_OBJECTS_H
#define __KEY_OBJECTS_H

#include "../util/SGN_StrArray.h"
#include "../util/SGN_ObjArray.h"
#include "../util/SGN_KeyValues.h"

/*!\brief Pairs of a key string and its value as an SGN_Object instance.
 *
 *\author Yoshinori Tamada &lt;tamada at is.s.u-tokyo.ac.jp&gt;
 */
typedef struct {
#ifdef DOXY
private:
#endif
  SGN_StrArray * keys;
  SGN_ObjArray * objects;
} SGN_KeyObjects;

SGN_KeyObjects * SGN_KeyObjects_new();
SGN_KeyObjects * SGN_KeyObjects_new_KeyValues(const SGN_KeyValues * kv);

void         SGN_KeyObjects_delete(SGN_KeyObjects * this);
SGN_KeyObjects * SGN_KeyObjects_clone(const SGN_KeyObjects * this);
SGN_KeyObjects * SGN_KeyObjects_clear(SGN_KeyObjects * this);
size_t       SGN_KeyObjects_size(const SGN_KeyObjects * this);

SGN_KeyObjects * SGN_KeyObjects_add(SGN_KeyObjects * this, const char * key, SGN_Object * object);
SGN_Object * SGN_KeyObjects_get(SGN_KeyObjects * this, const char * key);
size_t       SGN_KeyObjects_find(const SGN_KeyObjects * this, const char * key);
const char * SGN_KeyObjects_get_key(const SGN_KeyObjects * this, size_t idx);
SGN_Object * SGN_KeyObjects_get_value(SGN_KeyObjects * this, size_t index);
const SGN_Object * SGN_KeyObjects_get_const_value(const SGN_KeyObjects * this, size_t index);

SGN_Type     SGN_KeyObjects_get_type(const SGN_KeyObjects * this, size_t index);


SGN_KeyObjects * SGN_KeyObjects_add_int(SGN_KeyObjects * this, const char * key, int value);
SGN_KeyObjects * SGN_KeyObjects_set_int(SGN_KeyObjects * this, const char * key, int value);
int          SGN_KeyObjects_get_int(const SGN_KeyObjects * this, const char * key, int def_value);
int          SGN_KeyObjects_get_int_at(const SGN_KeyObjects * this, size_t index, int def_value);

SGN_KeyObjects * SGN_KeyObjects_add_double(SGN_KeyObjects * this, const char * key, double value);
SGN_KeyObjects * SGN_KeyObjects_set_double(SGN_KeyObjects * this, const char * key, double value);
SGN_KeyObjects * SGN_KeyObjects_add_string(SGN_KeyObjects * this, const char * key, const char * value);

SGN_KeyObjects * SGN_KeyObjects_append_KeyObjects(SGN_KeyObjects * this, const SGN_KeyObjects * ko);
SGN_KeyObjects * SGN_KeyObjects_append_KeyValues(SGN_KeyObjects * this, const SGN_KeyValues * kv);

SGN_KeyObjects * SGN_KeyObjects_add_by_string(SGN_KeyObjects * this, const char * key,
									  const char * value, SGN_Type type);

SGN_KeyObjects * SGN_KeyObjects_remove_at(SGN_KeyObjects * this, size_t index);

void         SGN_KeyObjects_fprint(const SGN_KeyObjects * this, FILE * fp);
void         SGN_KeyObjects_check_validity(const SGN_KeyObjects * this);

#define SGN_KeyObjects_ERROR_VALUE SGN_StrArray_ERROR_VALUE

#endif /* __KEY_OBJECTS_H */

/* End of file.  Copyright (C) 2010  SiGN Project Members */
