/*
  sign/util/SGN_Memory.h : Memory utilities

  Copyright (C) 2008-2012 SiGN Project Members.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_MEMORY_H
#define __SGN_MEMORY_H

#include <stdio.h>
#include <stdlib.h>

#ifdef SIGN_DEBUG

#define malloc(x)      SGN_Memory_allocate(x, __FILE__, __LINE__)
#define realloc(x, y)  SGN_Memory_reallocate(x, y, __FILE__, __LINE__)
#define free(x)        SGN_Memory_free(x, __FILE__, __LINE__)

#endif /* SIGN_DEBUG */

void * SGN_Memory_allocate(size_t size, const char * file, int line);
void * SGN_Memory_reallocate(void * ptr, size_t size, const char * file, int line);
void   SGN_Memory_free(void * ptr, const char * file, int line);
void   SGN_Memory_show(FILE * fp);
int    SGN_Memory_num();

/*!\brief Allocates memory for \a n double values.
 *\ingroup SGN_Memory */
#define double_new(n) ((double *) malloc(sizeof(double) * n))
#define int_new(x)    ((int *) malloc(sizeof(int) * x))

#endif /* __SGN_MEMORY_H */

/* End of file.  Copyright (C) 2008-2012  SiGN Project Members */
