/*
  sign/util/SGN_StrArray.h : Expandable array for strings.

  Copyright (C) 2008 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_STR_ARRAY_H
#define __SGN_STR_ARRAY_H

#include "SGN_Array.h"
#include "SGN_IntArray.h"

typedef SGN_Array SGN_StrArray;

#ifdef DOXY


/*!\brief Expandable array for strings.
 *
 * \ref SGN_StrArray module provides functions to realize the automatically
 * expandable array for storing strings, i.e., char pointers.
 * This module is implemented as a set of wrapper functions for \ref SGN_Array
 * module.
 *
 * Do not use the SGN_StrArray structure directly.  To use \ref SGN_StrArray module,
 * use SGN_StrArray_new() to generate the \ref SGN_StrArray instance, and use
 * SGN_StrArray_delete() to release the memory used by the instance when you no
 * longer need to use it.
 *
 *\sa StrMatrix
 *\author Yoshinori Tamada <tamada@is.s.u-tokyo.ac.jp> */
typedef struct {
  
} SGN_StrArray;

/*!\brief Value representing the error.
 *\memberof SGN_StrArray */
#define SGN_StrArray_ERROR_VALUE

#else
#define SGN_StrArray_ERROR_VALUE  ((size_t) -1)

#endif /* DOXY */

SGN_StrArray * SGN_StrArray_new();
SGN_StrArray * SGN_StrArray_news(size_t size);
void           SGN_StrArray_delete(SGN_StrArray * this);
void           SGN_StrArray_delete_all(SGN_StrArray * this);
void           SGN_StrArray_delete_v(void * this);
void           SGN_StrArray_clear(SGN_StrArray * this);
void           SGN_StrArray_clear_all(SGN_StrArray * this);
SGN_StrArray * SGN_StrArray_new_ar(size_t ar_size);
void           SGN_StrArray_delete_ar_all(SGN_StrArray * this, size_t ar_size);
SGN_StrArray * SGN_StrArray_clone(const SGN_StrArray * this);
SGN_StrArray * SGN_StrArray_clone_ar(const SGN_StrArray * this, size_t size);
SGN_StrArray * SGN_StrArray_add(SGN_StrArray * this, char * str);
SGN_StrArray * SGN_StrArray_add_nr(SGN_StrArray * this, char * str);
SGN_StrArray * SGN_StrArray_add_copy(SGN_StrArray * this, const char * str);
size_t         SGN_StrArray_size(const SGN_StrArray * tthis);
char *         SGN_StrArray_get(const SGN_StrArray * this, size_t index);
SGN_StrArray * SGN_StrArray_set(SGN_StrArray * this, size_t index, char * str);
SGN_StrArray * SGN_StrArray_set_size(SGN_StrArray * this, size_t size);
void           SGN_StrArray_remove(SGN_StrArray * this, size_t index);
void           SGN_StrArray_remove_elm(SGN_StrArray * this, size_t index);
size_t         SGN_StrArray_find(const SGN_StrArray * this, const char * str);
size_t         SGN_StrArray_find_case(const SGN_StrArray * this, const char * str);
size_t         SGN_StrArray_find_add(SGN_StrArray * this, const char * str);
SGN_IntArray * SGN_StrArray_collect_indexes(const SGN_StrArray * this, const char * str);
void           SGN_StrArray_copy_all(const SGN_StrArray * this, SGN_StrArray * dst);
void           SGN_StrArray_fprint_v(const void * v, FILE * fp);

#ifdef USE_MPI
#include <mpi.h>
void SGN_StrArray_MPI_Bcast(SGN_StrArray ** this, int root, MPI_Comm comm);
void SGN_StrArray_ar_MPI_Bcast(SGN_StrArray ** this, size_t * size, 
						   int root, MPI_Comm comm);
#endif /* USE_MPI */

#endif /* __SGN_STR_ARRAH_H */

/* End of file.  Copyright (C) 2008 SiGN Project Membets */
