/*
  sign/util/SGN_Timer.h : Timer functions

  Copyright (C) 2008-2012 SiGN Project Members

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_TIMER_H
#define __SGN_TIMER_H

#include <time.h>
#include <stdio.h>
#include <sys/time.h>

/*!\brief Timer types.
 *
 *\ingroup Timer */
typedef enum {
  SGN_Timer_NORMAL,
  SGN_Timer_CLOCK,
} SGN_Timer_type_t;

typedef struct SGN_Timer_t SGN_Timer;

SGN_Timer * SGN_Timer_new();
SGN_Timer * SGN_Timer_new_ar(size_t size);
void    SGN_Timer_delete(SGN_Timer * timer);
void    SGN_Timer_set_type(SGN_Timer * timer, SGN_Timer_type_t type);
SGN_Timer_type_t
        SGN_Timer_get_type(const SGN_Timer * timer);

void    SGN_Timer_start(SGN_Timer * timer);
void    SGN_Timer_start_type(SGN_Timer * timer, SGN_Timer_type_t type);
void    SGN_Timer_stop(SGN_Timer * timer);
void    SGN_Timer_clear(SGN_Timer * timer);
void    SGN_Timer_clear_start(SGN_Timer * timer);

double  SGN_Timer_get(const SGN_Timer * timer);
double  SGN_Timer_get_avg_lap(const SGN_Timer * timer);
void    SGN_Timer_print_DDHHMMSS(FILE * fp, SGN_Timer * timer);
void    SGN_Timer_print_DDHHMMSS_sec(FILE * fp, double seconds);

#endif /* __SGN_TIMER_H */

/* End of file.  Copyright (C) 2008-2012  SiGN Project Mbmers  */
