/*
  util/SGN_Util.h : Utility Functions

  Copyright (C) 2008-2011 SiGN Project Members.

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.

  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This is a part of SiGN Library.
  Visit <http://sign.hgc.jp/> for more information.

  Written by Yoshinori Tamada <tamada at is.s.u-tokyo.ac.jp>
*/

#ifndef __SGN_UTIL_H
#define __SGN_UTIL_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <stdint.h>

#include "../math/SGN_RNG.h"

void   SGN_Util_print_size_t_comma(FILE * fp, size_t value);
void   SGN_Util_print_int_comma(FILE * fp, int value);
double SGN_Util_timeval_span(const struct timeval * start_time,
						 const struct timeval * end_time);
void   SGN_Util_print_timeval_span(FILE * fp,
							   const struct timeval * start_time,
							   const struct timeval * end_time);
void SGN_Util_sec_to_day_hour_min_sec(double in_sec,
								  int * day, int * hour, int * min, int * sec);
void SGN_Util_print_sec_in_day_hour_min_sec(FILE * fp, double in_sec);

const char * SGN_Util_get_svn_rev(const char * svnrev);

div_t SGN_Util_job_split(int total, int procs, int rank, int * start, int * size);
int   SGN_Util_job_rank(int quot, int rem, int job);
int SGN_Util_job_start(int quot, int rem, int rank);

void SGN_Util_random_sample(int max, int num, int * buff);
void SGN_Util_random_sample_fix(int max, int num, int * buff, int fix);
void SGN_Util_random_sample_set(int * set, int size, int num, int * buff);
void SGN_Util_random_perm(int * buff, int num);

void SGN_Util_random_sample_ushort(unsigned short max, int num,
								   unsigned short * buff);
void SGN_Util_random_sample_ushort_RNG(unsigned short max, int num,
									   unsigned short * buff, SGN_RNG * rng);
void SGN_Util_random_sample_fix_ushort(unsigned short max, int num,
								   unsigned short * buff, int fix);
void SGN_Util_random_sample_fix_ushort_RNG(unsigned short max, int num,
										   unsigned short * buff, int fix,
										   SGN_RNG * rng);

/* Array Utilities */
void SGN_Util_int_array_sort_asc(int * ar, size_t size);
void SGN_Util_int_array_sort_dsc(int * ar, size_t size);


uint32_t SGN_Util_badrnd32();
void SGN_Util_badrnd32_seed(uint32_t seed);

#endif /* __SGN_UTIL_H */

/* End of file.  Copyright (C) 2008-2011 SiGN Project Members */
